module LatLonDistance exposing (lat_lon_distance, LatLon)

type alias LatLon = { lat : Float, lon : Float }

lat_lon_distance : LatLon -> LatLon -> Float
lat_lon_distance p1 p2 =
  let
    a = 6378137.0
    b = 6356752.314245
    f = 1 / 298.257223563
    lat1 = degrees p1.lat
    lat2 = degrees p2.lat
    lon1 = degrees p1.lon
    lon2 = degrees p2.lon

    dlon = (lon2 - lon1)

    tanU1 = (1-f) * (tan lat1)
    cosU1 = 1 / (sqrt (1 + tanU1 * tanU1))
    sinU1 = tanU1 * cosU1

    tanU2 = (1-f) * (tan lat2)
    cosU2 = 1 / (sqrt (1 + tanU2*tanU2))
    sinU2 = tanU2 * cosU2

    approx lon =
      let
        sinlon = sin lon
        coslon = cos lon
        sinSqsigma = (cosU2*sinlon) * (cosU2*sinlon) + (cosU1*sinU2-sinU1*cosU2*coslon) ^ 2
        sinsigma = sqrt sinSqsigma
        cossigma = sinU1*sinU2 + cosU1*cosU2*coslon
        sigma = atan2 sinsigma cossigma
        sinalpha = cosU1 * cosU2 * sinlon / sinsigma
        cosSqalpha = 1 - sinalpha*sinalpha
        cos2sigma_m = cossigma - 2*sinU1*sinU2/cosSqalpha
        c = f/16*cosSqalpha*(4+f*(4-3*cosSqalpha))
        lon_ = dlon + (1-c) * f * sinalpha * (sigma + c*sinsigma*(cos2sigma_m+c*cossigma*(-1+2*cos2sigma_m*cos2sigma_m)))
        
        uSq = cosSqalpha * (a*a - b*b) / (b*b)
        biga = 1 + uSq/16384*(4096+uSq*(-768+uSq*(320-175*uSq)))
        bigb = uSq/1024 * (256+uSq*(-128+uSq*(74-47*uSq)))
        deltasigma = bigb*sinsigma*(cos2sigma_m+bigb/4*(cossigma*(-1+2*cos2sigma_m*cos2sigma_m)-bigb/6*cos2sigma_m*(-3+4*sinsigma*sinsigma)*(-3+4*cos2sigma_m*cos2sigma_m)))
        
        s = b*biga*(sigma-deltasigma)
        
      in
        if abs (lon - lon_) > 1e-12 then (approx lon_) else s
  in
    approx dlon